<?php
/**
 * ###############################################
 *
 * SWIFT Framework
 * _______________________________________________
 *
 * @author         Varun Shoor
 *
 * @package        SWIFT
 * @copyright      Copyright (c) 2001-2012, Kayako
 * @license        http://www.kayako.com/license
 * @link           http://www.kayako.com
 *
 * ###############################################
 */

/**
 * Controller Class for Ajax Request
 *
 * @author Rahul Bhattacharya
 */
class SWIFT_SugarCRMRestAPIRequest extends SWIFT_Library
{
	/**
	 * Used for storing the response of the request parameter
	 *
	 * @access protected
	 * @var string
	 *
	 * @author Rahul Bhattacharya
	 */
	protected $_response = 'fail';

	/**
	 * Constructor
	 *
	 * return boolean True on Success, or False Otherwise
	 *
	 * @author Rahul Bhattacharya
	 */
	public function __construct()
	{
		parent::__construct();
	}


	/**
	 * Destructor
	 *
	 * return boolean True on Success, or False Otherwise
	 *
	 * @author Rahul Bhattacharya
	 */
	public function __destruct()
	{
		parent::__destruct();
	}

	/**
	 * Get the login session response based on the HTTP request
	 *
	 * @param string $_userName
	 * @param string $_password
	 * @param string $_url
	 *
	 * @return object response
	 *
	 * @author Rahul Bhattacharya
	 */
	public function LoginRequest($_userName, $_password, $_url)
	{
		$_loginParameters = array(
			'user_auth'        => array(
				'user_name' => $_userName,
				'password'  => md5($_password),
				'version'   => '1'
			),
			'application_name' => 'RestTest',
			'name_value_list ' => array(),
		);

		$_requestParameters = array(
			'url'        => $_url,
			'method'     => 'login',
			'parameters' => $_loginParameters
		);

		$_loginResult = $this->CurlResponse($_requestParameters);

		if (!empty($_loginResult->id)) {
			return $_loginResult->id;
		} else {
			return $this->_response;
		}
	}

	/**
	 * Fetch the list of accounts from SugarCrm using REST API
	 *
	 * @param string $_sessionId
	 * @param string $_url
	 *
	 * @return object response
	 *
	 * @author Rahul Bhattacharya
	 */
	public function FetchAccounts($_sessionId, $_url)
	{
		$_getEntryListParameters = array(
			'session'                   => $_sessionId,
			'module_name'               => 'Accounts',
			'query'                     => "",
			'order_by'                  => "",
			'offset'                    => '0',
			'select_fields'             => array(
				'id',
				'name',
				'billing_address_street',
				'billing_address_city',
				'billing_address_state',
				'billing_address_postalcode',
				'billing_address_country',
				'phone_office',
				'website',
				'date_entered',
				'date_modified',
			),

			//A list of link names and the fields to be returned for each link name.
			'link_name_to_fields_array' => array(),
			'max_results'               => '999',
			//To exclude deleted records
			'deleted'                   => '0',
			//If only records marked as favorites should be returned.
			'Favorites'                 => false,
		);

		$_requestParameters = array(
			'url'        => $_url,
			'method'     => 'get_entry_list',
			'parameters' => $_getEntryListParameters
		);

		return $this->CurlResponse($_requestParameters);
	}

	/**
	 * Fetch the list of contacts from SugarCrm using REST API
	 *
	 * @author Rahul Bhattacharya
	 *
	 * @param string $_sessionId
	 * @param string $_url
	 * @param string $_accountId
	 *
	 * @return object response
	 */
	public function FetchContacts($_sessionId, $_url, $_accountId)
	{
		$_getEntryListParameters = array(
			'session'              => $_sessionId,
			'module_name'          => 'Accounts',
			'module_id'            => $_accountId,
			'link_field_name'      => 'contacts',
			'related_module_query' => "",
			'related_fields'       => array(
				'id',
				'name',
				'email1',
				'phone_work',
				'title',

			),
			'max_results'          => '999',
		);

		$_requestParameters = array(
			'url'        => $_url,
			'method'     => 'get_relationships',
			'parameters' => $_getEntryListParameters
		);

		return $this->CurlResponse($_requestParameters);
	}

	/**
	 * Fetch the list of leads from SugarCrm using REST API
	 *
	 * @author Rahul Bhattacharya
	 *
	 * @param string $_sessionId
	 * @param string $_url
	 * @param string $_accountId
	 * @param bool   $_isUser
	 *
	 * @return object response
	 */
	public function FetchLeads($_sessionId, $_url, $_accountId, $_isUser = false)
	{
		$_idType     = 'account_id';
		$_moduleName = 'Accounts';

		if ($_isUser)
        {
			$_idType     = 'user_id';
			$_moduleName = 'Contacts';
		}

        if (empty($_accountId))
        {
            return false;
        }

		$_getEntryListParameters = array(
			'session'              => $_sessionId,
			'module_name'          => $_moduleName,
			$_idType               => $_accountId,
			'link_field_name'      => 'leads',
			'related_module_query' => "",
			'related_fields'       => array(
				'id',
				'name',
				'lead_source',
				'phone_work',
				'assigned_user_name',
			),
			'max_results'          => '999',
		);

        $_requestParameters = array(
			'url'        => $_url,
			'method'     => 'get_relationships',
			'parameters' => $_getEntryListParameters
		);

		return $this->CurlResponse($_requestParameters);
	}

	/**
	 * Fetch the list of opportunities of particular account from SugarCrm using REST API
	 *
	 * @author Rahul Bhattacharya
	 *
	 * @param string    $_sessionId
	 * @param string    $_url
	 * @param string    $_accountId
	 * @param bool      $_isUser
	 *
	 * @return object response
	 */
	public function FetchOpportunities($_sessionId, $_url, $_accountId, $_isUser = false)
	{
		$_idType     = 'account_id';
		$_moduleName = 'Accounts';

		if ($_isUser)
        {
			$_idType     = 'user_id';
			$_moduleName = 'Contacts';
		}

		$_getEntryListParameters = array(
			'session'              => $_sessionId,
			'module_name'          => $_moduleName,
			$_idType               => $_accountId,
			'link_field_name'      => 'opportunities',
			'related_module_query' => "",
			'related_fields'       => array(
				'id',
				'name',
				'account_name',
				'sales_stage',
				'close',
				'amount_usdollar',
				'assigned_user_name',
			),
			'max_results'          => '999',
		);

		$_requestParameters = array(
			'url'        => $_url,
			'method'     => 'get_relationships',
			'parameters' => $_getEntryListParameters
		);

		return $this->CurlResponse($_requestParameters);
	}

	/**
	 * Create Lead
	 *
	 * @author Rahul Bhattacharya
	 * @param SWIFT_Ticket $_SWIFT_TicketObject
	 * @param string       $_email Email Id
	 * @param string       $_sessionId Session Id
	 * @param string       $_url REST API URL
	 * @return string $_response on Success, "false" otherwise
	 */
	public function CreateLead(SWIFT_Ticket $_SWIFT_TicketObject, $_email = '', $_sessionId, $_url)
	{
		$_SWIFT_UserObject = $_SWIFT_TicketObject->GetUserObject();

		$_SWIFT_UserOrganizationObject = $_SWIFT_TicketObject->GetUserOrganizationObject();

		$_record = new stdclass();
		$_record->LastName = $_SWIFT_UserObject->GetProperty('fullname');
		$_record->Description = $_SWIFT_TicketObject->GetLastPostContents();
		$_record->Title = $_SWIFT_UserObject->GetProperty('userdesignation');
		$_record->Phone = $_SWIFT_UserObject->GetProperty('phone');
		$_record->Id = "Ticket Id" . $_SWIFT_TicketObject->GetProperty('ticketmaskid');

		$_company = $_SWIFT_UserOrganizationObject ? $_SWIFT_UserOrganizationObject->GetProperty('organizationname') : "";

		if (!empty($_company))
		{
			$_record->Company = $_company;
		} else {
			$_record->Company = '';
		}

		if (!empty($_email))
		{
			$_record->Email = $_email;
		}

		$_response = $this->Create($_record, $_sessionId, $_url);

		if (isset($_response->id))
		{
			$this->Load->Model('TicketToLead:SugarCRMTicketLeadMap');

			$this->SugarCRMTicketLeadMap->InsertLead($_SWIFT_TicketObject->GetProperty('ticketid'), $_response->id);

			return 'Ticket has been successfully converted to Lead (Lead Id: ' . $_response->id . ' )';

		} else if (empty($_response->id)) {

			return "Error : Please try again. ";

		} else {
			return false;
		}
	}

	/**
	 *
	 *
	 * @author Rahul Bhattacharya
	 * @param array $_record
	 * @param string $_sessionId Session Id
	 * @param string $_url REST API URL
	 * @return bool "true" on Success, "false" otherwise
	 * @throws SWIFT_Exception If the Class is not Loaded
	 */
	public function Create($_record, $_sessionId, $_url)
	{
		if (!$this->GetIsClassLoaded()) {
			throw new SWIFT_Exception(__CLASS__ . ':  ' . SWIFT_CLASSNOTLOADED);
		}

		$_setEntryParameters = array(
			"session" => $_sessionId,
			"module_name" => "Leads",
			"name_value_list" => array(
				array(
					"name" => "last_name",
					"value" => $_record->LastName
				),
				array(
					"name" => "lead_source_description",
					"value" => $_record->Description
				),
				array(
					"name" => "title",
					"value" => $_record->Title
				),
				array(
					"name" => "phone_mobile",
					"value" => $_record->Phone
				),
				array(
					"name" => "email1",
					"value" => $_record->Email
				),
				array(
					"name" => "account_name",
					"value" => $_record->Company
				),
				array(
					"name" => "status_description",
					"value" => $_record->Id
				),
			),
		);

		$_requestParameters = array(
			'url'        => $_url,
			'method'     => 'set_entry',
			'parameters' => $_setEntryParameters
		);

		return $this->CurlResponse($_requestParameters);
	}

	/**
	 * Get the cURL HTTP response based on the HTTP request
	 *
	 * @author Rahul Bhattacharya
	 * @param array $_requestOption
	 * @return string $_response on Success, "false" otherwise
	 * @throws Exception
	 */
	public function CurlResponse($_requestOption)
	{
		$_curl_request = curl_init();

		curl_setopt($_curl_request, CURLOPT_URL, $_requestOption['url']);
		curl_setopt($_curl_request, CURLOPT_POST, 1);
		curl_setopt($_curl_request, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
		curl_setopt($_curl_request, CURLOPT_HEADER, 1);
		curl_setopt($_curl_request, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($_curl_request, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($_curl_request, CURLOPT_FOLLOWLOCATION, 0);

		$_jsonEncodedData = json_encode($_requestOption['parameters']);

		$_postArray = array(
			"method"        => $_requestOption['method'],
			"input_type"    => "JSON",
			"response_type" => "JSON",
			"rest_data"     => $_jsonEncodedData
		);

		curl_setopt($_curl_request, CURLOPT_POSTFIELDS, $_postArray);
		$_result        = curl_exec($_curl_request);
		$_result        = explode("\r\n\r\n", $_result, 2);
		$_content       = json_decode($_result[1]);
		$_error_message = curl_error($_curl_request);
		$_error_number  = curl_errno($_curl_request);
		$_http_code     = curl_getinfo($_curl_request, CURLINFO_HTTP_CODE);
		curl_close($_curl_request);

		$_header['errno']     = $_error_number;
		$_header['errormsg']  = $_error_message;
		$_header['http_code'] = $_http_code;
		$_header['content']   = $_content;

		try {
			if (!$_header['http_code'] == 200 || !$_header['http_code'] == 400) {
				throw new SWIFT_Exception(SWIFT_RESTServer::HTTP_BADREQUEST);

				return false;
			}

			return $_header['content'];
		} catch (Exception $_e) {
			echo $_e->getMessage();
		}

        return false;
	}
}
